#include "HUSKYLENS.h"
#include <Servo.h>
#include <PID_v1.h>

Servo pan;  // create servo object to control the panning servo
Servo str;  // create servo object to control the steering servo

//Define Variables we'll be connecting to
double Setpoint1, Input1, Output1;

//Specify the links and initial tuning parameters
double Kp1 = 0.10  , Ki1 = 1.50, Kd1 = 0.000;
PID panPID(&Input1, &Output1, &Setpoint1, Kp1, Ki1, Kd1, DIRECT);

int panlimit = 50;
int pancenter = 95;
int strcenter = 90;
int str_com = 90;
int xo = 160;
int xt = 160;
double xo_ave = 0;
double xt_ave = 0;

HUSKYLENS huskylens;
//HUSKYLENS green line >> SDA; blue line >> SCL

void setup() {

  pinMode(5, OUTPUT); // TB67H450FNG IN2 用出力ピン
  pinMode(6, OUTPUT); // TB67H450FNG IN1 用出力ピン
  pinMode(7, OUTPUT); // TB67H450FNG VREF用出力ピン

  Serial.begin(115200);
  Wire.begin();
  while (!huskylens.begin(Wire))
  {
    Serial.println(F("Begin failed!"));
    Serial.println(F("1.Please recheck the \"Protocol Type\" in HUSKYLENS (General Settings>>Protocol Type>>I2C)"));
    Serial.println(F("2.Please recheck the connection."));
    digitalWrite(7, LOW);
    delay(100);
  }

  pan.attach(9);  // attaches the servo on pin  9 to the panning servo object
  str.attach(10); // attaches the servo on pin 11 to the steering servo object

  //initialize the variables we're linked to
  Input1 = xo_ave;
  Setpoint1 = 160;

  panPID.SetOutputLimits(-1 * panlimit, panlimit);
  panPID.SetSampleTime(30);

  //turn the PID on
  panPID.SetMode(AUTOMATIC);

  // Initialize of the vehicle status
  pan.write(pancenter);
  str.write(strcenter);
  delay(500);

  // Start up the vehicle
  digitalWrite(5, LOW);  //IN2
  analogWrite (6, 160);  //IN1
  digitalWrite(7, HIGH);
  delay(1000);
}

void loop() {
  if (!huskylens.request()) {
    Serial.println(F("Fail to request data from HUSKYLENS, recheck the connection!"));
    digitalWrite(7, LOW);
  }

  else if (!huskylens.isLearned()) {
    Serial.println(F("Nothing learned, press learn button on HUSKYLENS to learn one!"));
    digitalWrite(7, LOW);
  }

  else if (!huskylens.available()) {
    Serial.println(F("No arrow appears on the screen!"));
    digitalWrite(7, LOW);
    finding_line();

  }

  while (huskylens.available())
  {
    xo_ave = 0;
    xt_ave = 0;

    int ave = 30;

    for (int i = 0; i < ave ; i++) {

      HUSKYLENSResult result = huskylens.read();
      if (result.command == COMMAND_RETURN_ARROW) {

        xo = result.xOrigin;
        xt = result.xTarget;

      }

      xo_ave = xo_ave + xo;
      xt_ave = xt_ave + xt;

    }
    xo_ave = xo_ave / ave;
    xt_ave = xt_ave / ave;

    digitalWrite(7, HIGH); //enabling traction motor
    finding_line();
    drive_motor();

    // scaling of panning angele to steering angle
    str_com = map (Output1, -1 * panlimit, panlimit, -30, 30);
    
    // add feedfoward value of line direction
    str_com = str_com + map(xt_ave, 320, 0, -16, 16);    
    
    str_com = constrain(str_com, -30, 30);               // limiting of steering angle
    str.write(str_com + strcenter);
  }
}

void finding_line() {
  Input1 = xo_ave ;
  panPID.Compute();
  
  // add feed foward value of line direction
  int pan_com = Output1 + map(xt_ave, 320, 0, -10, 10); 
     
  pan.write(pan_com + pancenter);
}

void drive_motor() {

  int acc_com = abs(str_com);
  acc_com = map(acc_com, 0, 30, 160, 70);
  acc_com = constrain(acc_com, 0, 255);

  digitalWrite(5, LOW);     //IN2
  analogWrite (6, acc_com); //IN1
}
